/**
 * 收费
 */

import { ORDERSTATE, PAYMENTTYPE, RECEIPTCONFIRMATION, WEICHARTPAYSTATE } from "../../config/enum";
import { TABLEENUM } from "../../data/models/model";
import { findOnce } from "../../data/select";
import { eccEnumValue } from "../../util/verificationEnum";
import * as path from "path";
const WxPay = require('wechatpay-node-v3');
import * as fs from 'fs';
import { BizError } from "../../util/bizError";
import { ERRORENUM } from "../../config/errorEnum";
import { generateWXOrderId, successResult } from "../../tools/system";
import { updateOneData } from "../../data/update";
import moment = require("moment");
import { createOrder } from "./order";

const AppId = 'wx96c0cf11b3babcf6';
const MchId = '1614605859';
const V3Key = 'c0338804d0c6a60cced9cad83a9a0402';
const pay = new WxPay({
    appid: AppId,
    mchid: MchId,
    publicKey: fs.readFileSync( path.join(__dirname.substring(0,__dirname.indexOf("out")), "keys", 'apiclient_cert.pem'  ) ), // 公钥
    privateKey: fs.readFileSync(  path.join(__dirname.substring(0,__dirname.indexOf("out")), "keys", 'apiclient_key.pem'  ) ), // 秘钥
});


/**
 * native下单接口【前端调用】
 * @param param0 {id订单列表的id}
 * @returns 
 */
export async function placeAnOrder({id}) {
    let orderInfo = await findOnce(TABLEENUM.订单表, {id}, ["id", "loginId", "userId", "memberCategory", "money", "orderCycleStart", "orderCycleEnd","weChartState", "state"]);
    if (!orderInfo || !orderInfo.id) throw new BizError(ERRORENUM.订单不存在);
    if (orderInfo.state == ORDERSTATE.已支付) throw new BizError(ERRORENUM.该订单已支付);
    if (orderInfo.weChartState != WEICHARTPAYSTATE.未提交 ) throw new BizError(ERRORENUM.该订单正在支付);
    /**生成 out_trade_no */
    let out_trade_no = generateWXOrderId(orderInfo.memberCategory, orderInfo.userId);
    let desc = `${orderInfo.loginId}_${moment(orderInfo.orderCycleStart).format("YYYY-MM-DD")}至${moment(orderInfo.orderCycleEnd).format("YYYY-MM-DD")}`;

    /**请求参数 */
    const Param = {
        appid: AppId,
        mchid: MchId,
        description:desc,
        out_trade_no,//订单号
        amount:{//金额
            // total:orderInfo.money
            total:1//单位分  金额要乘100
        },
        notify_url:'https://yydata.mynatapp.cc/pay/wechart/callback',//订单回调
    };
    const result = await pay.transactions_native(Param);
    if (result.status != 200) throw new BizError(ERRORENUM.调起微信支付失败);

    await updateOneData(TABLEENUM.订单表, {id}, {weChartPR:out_trade_no, paymentMethod:PAYMENTTYPE.微信支付, weChartState:WEICHARTPAYSTATE.未支付 });
    return {code_url:result.code_url, weChartPR:out_trade_no};
}


/**
 * 订单查询状态【前端调用】
 */
export async function orderState({id, weChartPR}) {
    let orderInfo = await findOnce(TABLEENUM.订单表, {id, weChartPR}, ["id"]);
    if (!orderInfo || !orderInfo.id) throw new BizError(ERRORENUM.订单不存在);

    if (orderInfo.state == ORDERSTATE.已支付) {
        return {state:WEICHARTPAYSTATE.已支付, msg:"已支付"};//已支付
    }
    const result = await pay.query({out_trade_no: weChartPR});
    if (result.status != 200 && result.status != 204 ) throw new BizError(ERRORENUM.查看微信支付状态失败);
    if (result.trade_state == 'CLOSED' ) {
        await updateOneData(TABLEENUM.订单表, {id}, { weChartState:WEICHARTPAYSTATE.未支付 });
        return {state:WEICHARTPAYSTATE.未提交, msg:"订单已关闭"};
    }

    return {state:WEICHARTPAYSTATE.未支付, msg:result.trade_state_desc};
}


/**
 * 关闭订单【前端调用】
 */
export async function orderClose({id, weChartPR}) {
    let orderInfo = await findOnce(TABLEENUM.订单表, {id, weChartPR}, ["id"]);
    if (!orderInfo || !orderInfo.id) throw new BizError(ERRORENUM.订单不存在);

    await pay.close(weChartPR);

    await updateOneData(TABLEENUM.订单表, {id}, { weChartState:WEICHARTPAYSTATE.未提交 });

    return successResult();
}



/**
 * 微信回调函数【微信用】
 */
export async function payCallback(body) {
    if (body.resource && body.summary == "支付成功") {
        const wechartCallbackData = pay.decipher_gcm(body.resource.ciphertext, body.resource.associated_data, body.resource.nonce, V3Key);
        if (wechartCallbackData.trade_state == "SUCCESS") {
            /**支付成功 */
            let oldInfo = await findOnce(TABLEENUM.订单表, {weChartPR:wechartCallbackData.out_trade_no});
            if (!oldInfo || !oldInfo.id) throw new BizError(ERRORENUM.目标数据不存在);
            let updateInfo:any = {
                paymentNum:wechartCallbackData.transaction_id, 
                state:ORDERSTATE.已支付, 
                paymentMethod:PAYMENTTYPE.微信支付
            };
            if (!oldInfo.isFirst) {
                updateInfo.confirmReceipt = RECEIPTCONFIRMATION.待确认;
            }
            await updateOneData(TABLEENUM.订单表, {weChartPR:wechartCallbackData.out_trade_no}, updateInfo);
            return {
                code:"SUCCESS",
                message:"成功"
            }
        } else {
            throw new BizError(ERRORENUM.支付失败, wechartCallbackData.trade_state_desc);
        }
    } else {
        throw new BizError(ERRORENUM.微信支付回调错误, body.summary);
    }
}